@@ -147,7 +147,7 @@ set_props(const struct libevdev *dev, int fd, struct uinput_user_dev *uidev)
 			 * ioctl is called on an already created device. The
 			 * last two can't happen here.
 			 */
-			if (errno == -EINVAL)
+			if (errno == EINVAL)
 				rc = 0;
 			break;
 		}
@@ -198,8 +198,17 @@ fetch_syspath_and_devnode(struct libevdev_uinput *uinput_dev)
 {
 	struct dirent **namelist;
 	int ndev, i;
+	int rc;
+	char buf[sizeof(SYS_INPUT_DIR) + 64] = SYS_INPUT_DIR;
 
-	/* FIXME: use new ioctl() here once kernel supports it */
+	rc = ioctl(uinput_dev->fd,
+		   UI_GET_SYSNAME(sizeof(buf) - strlen(SYS_INPUT_DIR)),
+		   &buf[strlen(SYS_INPUT_DIR)]);
+	if (rc != -1) {
+		uinput_dev->syspath = strdup(buf);
+		uinput_dev->devnode = fetch_device_node(buf);
+		return 0;
+	}
 
 	ndev = scandir(SYS_INPUT_DIR, &namelist, is_input_device, alphasort);
 	if (ndev <= 0)
@@ -207,11 +216,14 @@ fetch_syspath_and_devnode(struct libevdev_uinput *uinput_dev)
 
 	for (i = 0; i < ndev; i++) {
 		int fd, len;
-		char buf[sizeof(SYS_INPUT_DIR) + 64];
 		struct stat st;
 
-		strcpy(buf, SYS_INPUT_DIR);
-		strcat(buf, namelist[i]->d_name);
+		rc = snprintf(buf, sizeof(buf), "%s%s/name",
+			      SYS_INPUT_DIR,
+			      namelist[i]->d_name);
+		if (rc < 0 || (size_t)rc >= sizeof(buf)) {
+			continue;
+		}
 
 		if (stat(buf, &st) == -1)
 			continue;
@@ -222,7 +234,6 @@ fetch_syspath_and_devnode(struct libevdev_uinput *uinput_dev)
 			continue;
 
 		/* created within time frame */
-		strcat(buf, "/name");
 		fd = open(buf, O_RDONLY);
 		if (fd < 0)
 			continue;
@@ -236,11 +247,17 @@ fetch_syspath_and_devnode(struct libevdev_uinput *uinput_dev)
 		if (strcmp(buf, uinput_dev->name) == 0) {
 			if (uinput_dev->syspath) {
 				/* FIXME: could descend into bit comparison here */
-				log_info("multiple identical devices found. syspath is unreliable\n");
+				log_info(NULL, "multiple identical devices found. syspath is unreliable\n");
 				break;
 			} else {
-				strcpy(buf, SYS_INPUT_DIR);
-				strcat(buf, namelist[i]->d_name);
+				rc = snprintf(buf, sizeof(buf), "%s%s",
+					      SYS_INPUT_DIR,
+					      namelist[i]->d_name);
+				if (rc < 0 || (size_t)rc >= sizeof(buf)) {
+					log_error(NULL, "Invalid syspath, syspath is unreliable\n");
+					break;
+				}
+
 				uinput_dev->syspath = strdup(buf);
 				uinput_dev->devnode = fetch_device_node(buf);
 			}
@@ -262,6 +279,7 @@ libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct li
 	int rc;
 	struct uinput_user_dev uidev;
 	struct libevdev_uinput *new_device;
+	int close_fd_on_error = (fd == LIBEVDEV_UINPUT_OPEN_MANAGED);
 
 	new_device = alloc_uinput_device(libevdev_get_name(dev));
 	if (!new_device)
@@ -274,7 +292,7 @@ libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct li
 
 		new_device->fd_is_managed = 1;
 	} else if (fd < 0) {
-		log_bug("Invalid fd %d\n", fd);
+		log_bug(NULL, "Invalid fd %d\n", fd);
 		errno = EBADF;
 		goto error;
 	}
@@ -317,7 +335,7 @@ libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct li
 	new_device->fd = fd;
 
 	if (fetch_syspath_and_devnode(new_device) == -1) {
-		log_error("unable to fetch syspath or device node.\n");
+		log_error(NULL, "unable to fetch syspath or device node.\n");
 		errno = ENODEV;
 		goto error;
 	}
@@ -327,8 +345,11 @@ libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct li
 	return 0;
 
 error:
+	rc = -errno;
 	libevdev_uinput_destroy(new_device);
-	return -errno;
+	if (fd != -1 && close_fd_on_error)
+		close(fd);
+	return rc;
 }
 
 LIBEVDEV_EXPORT void
@@ -337,7 +358,7 @@ libevdev_uinput_destroy(struct libevdev_uinput *uinput_dev)
 	if (!uinput_dev)
 		return;
 
-	ioctl(uinput_dev->fd, UI_DEV_DESTROY, NULL);
+	(void)ioctl(uinput_dev->fd, UI_DEV_DESTROY, NULL);
 	if (uinput_dev->fd_is_managed)
 		close(uinput_dev->fd);
 	free(uinput_dev->syspath);
