@@ -38,83 +38,83 @@ struct libevdev_uinput;
  * help to create uinput devices that emulate libevdev devices. In the simplest
  * form it serves to duplicate an existing device:
  *
- * @code
- * int err;
- * int new_fd;
- * struct libevdev *dev;
- * struct libevdev_uinput *uidev;
- * struct input_event ev[2];
- *
- * err = libevdev_new_from_fd(&dev, fd);
- * if (err != 0)
- *     return err;
- *
- * uifd = open("/dev/uinput", O_RDWR);
- * if (uidev < 0)
- *     return -errno;
- *
- * err = libevdev_uinput_create_from_device(dev, uifd, &uidev);
- * if (err != 0)
- *     return err;
- *
- * // post a REL_X event
- * err = libevdev_uinput_write_event(uidev, EV_REL, REL_X, -1);
- * if (err != 0)
- *     return err;
- * libevdev_uinput_write_event(uidev, EV_SYN, SYN_REPORT, 0);
- * if (err != 0)
- *     return err;
- *
- * libevdev_uinput_destroy(uidev);
- * close(uifd);
- *
- * @endcode
+ @code
+ int err;
+ int new_fd;
+ struct libevdev *dev;
+ struct libevdev_uinput *uidev;
+ struct input_event ev[2];
+
+ err = libevdev_new_from_fd(&dev, fd);
+ if (err != 0)
+     return err;
+
+ uifd = open("/dev/uinput", O_RDWR);
+ if (uidev < 0)
+     return -errno;
+
+ err = libevdev_uinput_create_from_device(dev, uifd, &uidev);
+ if (err != 0)
+     return err;
+
+ // post a REL_X event
+ err = libevdev_uinput_write_event(uidev, EV_REL, REL_X, -1);
+ if (err != 0)
+     return err;
+ libevdev_uinput_write_event(uidev, EV_SYN, SYN_REPORT, 0);
+ if (err != 0)
+     return err;
+
+ libevdev_uinput_destroy(uidev);
+ close(uifd);
+
+ @endcode
  *
  * Alternatively, a device can be constructed from scratch:
  *
- * @code
- * int err;
- * struct libevdev *dev;
- * struct libevdev_uinput *uidev;
- *
- * dev = libevdev_new();
- * libevdev_set_name(dev, "test device");
- * libevdev_enable_event_type(dev, EV_REL);
- * libevdev_enable_event_code(dev, EV_REL, REL_X);
- * libevdev_enable_event_code(dev, EV_REL, REL_Y);
- * libevdev_enable_event_type(dev, EV_KEY);
- * libevdev_enable_event_code(dev, EV_KEY, BTN_LEFT);
- * libevdev_enable_event_code(dev, EV_KEY, BTN_MIDDLE);
- * libevdev_enable_event_code(dev, EV_KEY, BTN_RIGHT);
- *
- * err = libevdev_uinput_create_from_device(dev,
- *                                          LIBEVDEV_UINPUT_OPEN_MANAGED,
- *                                          &uidev);
- * if (err != 0)
- *     return err;
- *
- * // ... do something ...
- *
- * libevdev_uinput_destroy(uidev);
- *
- * @endcode
+ @code
+ int err;
+ struct libevdev *dev;
+ struct libevdev_uinput *uidev;
+
+ dev = libevdev_new();
+ libevdev_set_name(dev, "test device");
+ libevdev_enable_event_type(dev, EV_REL);
+ libevdev_enable_event_code(dev, EV_REL, REL_X);
+ libevdev_enable_event_code(dev, EV_REL, REL_Y);
+ libevdev_enable_event_type(dev, EV_KEY);
+ libevdev_enable_event_code(dev, EV_KEY, BTN_LEFT);
+ libevdev_enable_event_code(dev, EV_KEY, BTN_MIDDLE);
+ libevdev_enable_event_code(dev, EV_KEY, BTN_RIGHT);
+
+ err = libevdev_uinput_create_from_device(dev,
+                                          LIBEVDEV_UINPUT_OPEN_MANAGED,
+                                          &uidev);
+ if (err != 0)
+     return err;
+
+ // ... do something ...
+
+ libevdev_uinput_destroy(uidev);
+
+ @endcode
  */
 
 enum libevdev_uinput_open_mode {
 	/* intentionally -2 to avoid to avoid code like the below from accidentally working:
 		fd = open("/dev/uinput", O_RDWR); // fails, fd is -1
 		libevdev_uinput_create_from_device(dev, fd, &uidev); // may hide the error */
-	LIBEVDEV_UINPUT_OPEN_MANAGED = -2, /**< let libevdev open and close @c /dev/uinput */
+	LIBEVDEV_UINPUT_OPEN_MANAGED = -2  /**< let libevdev open and close @c /dev/uinput */
 };
 
 /**
  * @ingroup uinput
  *
- * Create a uinput device based on the libevdev device given. The uinput device
+ * Create a uinput device based on the given libevdev device. The uinput device
  * will be an exact copy of the libevdev device, minus the bits that uinput doesn't
  * allow to be set.
  *
- * If uinput_fd is LIBEVDEV_UINPUT_OPEN_MANAGED, libevdev_uinput_create_from_device()
+ * If uinput_fd is @ref LIBEVDEV_UINPUT_OPEN_MANAGED, libevdev_uinput_create_from_device()
  * will open @c /dev/uinput in read/write mode and manage the file descriptor.
  * Otherwise, uinput_fd must be opened by the caller and opened with the
  * appropriate permissions.
@@ -133,7 +133,7 @@ enum libevdev_uinput_open_mode {
  * source device.
  *
  * @param dev The device to duplicate
- * @param uinput_fd LIBEVDEV_UINPUT_OPEN_MANAGED or a file descriptor to @c /dev/uinput,
+ * @param uinput_fd @ref LIBEVDEV_UINPUT_OPEN_MANAGED or a file descriptor to @c /dev/uinput,
  * @param[out] uinput_dev The newly created libevdev device.
  *
  * @return 0 on success or a negative errno on failure. On failure, the value of
@@ -150,13 +150,11 @@ int libevdev_uinput_create_from_device(const struct libevdev *dev,
  *
  * Destroy a previously created uinput device and free associated memory.
  *
- * If the device was opened with LIBEVDEV_UINPUT_OPEN_MANAGED, libevdev_uinput_destroy()
- * also closes the file descriptor. Otherwise, the fd is left as-is and
- * must be closed by the caller.
+ * If the device was opened with @ref LIBEVDEV_UINPUT_OPEN_MANAGED,
+ * libevdev_uinput_destroy() also closes the file descriptor. Otherwise, the
+ * fd is left as-is and must be closed by the caller.
  *
  * @param uinput_dev A previously created uinput device.
- *
- * @return 0 on success or a negative errno on failure
  */
 void libevdev_uinput_destroy(struct libevdev_uinput *uinput_dev);
 
@@ -178,22 +176,20 @@ int libevdev_uinput_get_fd(const struct libevdev_uinput *uinput_dev);
 /**
  * @ingroup uinput
  *
- * Return the syspath representing this uinput device.
- * As of 3.11, the uinput kernel device does not
- * provide a way to get the syspath directly through uinput so libevdev must guess.
- * In some cases libevdev is unable to derive the syspath. If the running kernel
- * supports the UI_GET_SYSPATH ioctl, the syspath is retrieved through that and will
- * be reliable and not be NULL.
+ * Return the syspath representing this uinput device. If the UI_GET_SYSNAME
+ * ioctl not available, libevdev makes an educated guess.
+ * The UI_GET_SYSNAME ioctl is available since Linux 3.15.
  *
- * @note This function may return NULL. libevdev currently uses ctime and
- * the device name to guess devices. To avoid false positives, wait at least
- * wait at least 1.5s between creating devices that have the same name.
+ * @note This function may return NULL if UI_GET_SYSNAME is not available.
+ * In that case, libevdev uses ctime and the device name to guess devices.
+ * To avoid false positives, wait at least wait at least 1.5s between
+ * creating devices that have the same name.
  * @param uinput_dev A previously created uinput device.
- * @return The syspath for this device, including preceding /sys.
+ * @return The syspath for this device, including the preceding /sys
  *
  * @see libevdev_uinput_get_devnode
  */
-const char*libevdev_uinput_get_syspath(struct libevdev_uinput *uinput_dev);
+const char* libevdev_uinput_get_syspath(struct libevdev_uinput *uinput_dev);
 
 /**
  * @ingroup uinput
@@ -203,7 +199,7 @@ const char*libevdev_uinput_get_syspath(struct libevdev_uinput *uinput_dev);
  * This relies on libevdev_uinput_get_syspath() to provide a valid syspath.
  * See libevdev_uinput_get_syspath() for more details.
  *
- * @note This function may return NULL. libevdev currently has to guess the
+ * @note This function may return NULL. libevdev may have to guess the
  * syspath and the device node. See libevdev_uinput_get_syspath() for details.
  * @param uinput_dev A previously created uinput device.
  * @return The device node for this device, in the form of /dev/input/eventN
