@@ -7,7 +7,6 @@
 from __future__ import print_function
 import re
 import sys
-import argparse
 
 class Bits(object):
 	pass
@@ -72,21 +71,6 @@ def print_bits(bits, prefix):
 	print("};")
 	print("")
 
-def print_python_bits(bits, prefix):
-	if  not hasattr(bits, prefix):
-		return
-
-	print("%s_map = {" % (prefix))
-	for val, name in list(getattr(bits, prefix).items()):
-		print("	%d : \"%s\"," % (val, name))
-	if prefix == "key":
-		for val, name in list(getattr(bits, "btn").items()):
-			print("	%d : \"%s\"," % (val, name))
-	print("}")
-	print("for k, v in %s_map.items():" % (prefix))
-	print("	%s_map[v] = k" % (prefix))
-	print("")
-
 def print_map(bits):
 	print("static const char * const * const event_type_map[EV_MAX + 1] = {")
 
@@ -149,7 +133,10 @@ def print_lookup_table(bits):
 		print_lookup(bits, prefix[:-1].lower())
 	print("};")
 	print("")
-
+	print("static const struct name_entry prop_names[] = {")
+	print_lookup(bits, "input_prop")
+	print("};")
+	print("")
 
 def print_mapping_table(bits):
 	print("/* THIS FILE IS GENERATED, DO NOT EDIT */")
@@ -157,10 +144,6 @@ def print_mapping_table(bits):
 	print("#ifndef EVENT_NAMES_H")
 	print("#define EVENT_NAMES_H")
 	print("")
-	print("#ifndef SYN_MAX /* added in 3.12  */")
-	print("#define SYN_MAX 0xf")
-	print("#endif")
-	print("")
 
 	for prefix in prefixes:
 		if prefix == "BTN_":
@@ -172,27 +155,6 @@ def print_mapping_table(bits):
 
 	print("#endif /* EVENT_NAMES_H */")
 
-def print_python_mapping_table(bits):
-	print("# THIS FILE IS GENERATED, DO NOT EDIT")
-	print("")
-
-	for prefix in prefixes:
-		if prefix == "BTN_":
-			continue
-		print_python_bits(bits, prefix[:-1].lower())
-
-	print_python_map(bits)
-
-	print("def event_get_type_name(type):")
-	print("	return ev_map[type]")
-	print("")
-	print("")
-	print("def event_get_code_name(type, code):")
-	print("	if map.has_key(type) and map[type].has_key(code):")
-	print("		return map[type][code]")
-	print("	return 'UNKNOWN'")
-	print("")
-
 def parse_define(bits, line):
 	m = re.match(r"^#define\s+(\w+)\s+(\w+)", line)
 	if m == None:
@@ -230,16 +192,14 @@ def parse(fp):
 
 	return bits
 
-if __name__ == "__main__":
-	parser = argparse.ArgumentParser()
-	parser.add_argument("--output", default="c")
-	parser.add_argument('source', metavar="/path/to/linux/input.h",
-			    type=argparse.FileType('r'),
-			    help='source file to parse')
+def usage(prog):
+	print("Usage: %s /path/to/linux/input.h" % prog)
 
-	args = parser.parse_args(sys.argv[1:])
-	bits = parse(args.source)
-	if args.output == "python":
-		print_python_mapping_table(bits)
-	else:
+if __name__ == "__main__":
+	if len(sys.argv) != 2:
+		usage(sys.argv[0])
+		sys.exit(2)
+
+	with open(sys.argv[1]) as f:
+		bits = parse(f)
 		print_mapping_table(bits)
